<div class="legend-container">
    Highlight colors:
    <span class="legend-container-item"{% if labels.MATCH_TOOLTIP %} data-tooltip="{{ labels.MATCH_TOOLTIP }}"{% endif %}><span style="color: {{color_scheme.MATCH}};">■</span> {{ labels.MATCH }}</span>
    <span class="legend-container-item"{% if labels.SUBSTITUTION_TOOLTIP %} data-tooltip="{{ labels.SUBSTITUTION_TOOLTIP }}"{% endif %}><span style="color: {{color_scheme.SUB}};">■</span> {{ labels.SUBSTITUTION }}</span>
    <span class="legend-container-item"{% if labels.INSERTION_TOOLTIP %} data-tooltip="{{ labels.INSERTION_TOOLTIP }}"{% endif %}><span style="color: {{color_scheme.INS}};">■</span> {{ labels.INSERTION }}</span>
    <span class="legend-container-item"{% if labels.DELETION_TOOLTIP %} data-tooltip="{{ labels.DELETION_TOOLTIP }}"{% endif %}><span style="color: {{color_scheme.DEL}};">■</span> {{ labels.DELETION }}</span>
    <span class="legend-container-item"{% if labels.PADDING_TOOLTIP %} data-tooltip="{{ labels.PADDING_TOOLTIP }}"{% endif %}><span style="color: {{color_scheme.PAD}};">■</span> {{ labels.PADDING }}</span>
    <span class="legend-container-item"{% if labels.KEYWORD_TOOLTIP %} data-tooltip="{{ labels.KEYWORD_TOOLTIP }}"{% endif %}><span class="kw kw-start kw-end">{{ labels.KEYWORD }}</span></span>
</div>
